<?php 
namespace WCOA\classes\admin;
use Automattic\WooCommerce\Utilities\OrderUtil;
use Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController;

class OrderPage
{
	//rplc: woocommerce-order-approval, wcoa, WCOA
	public function __construct()
	{
		add_action( 'add_meta_boxes', array( &$this, 'woocommerce_metaboxes' ));
		add_action( 'woocommerce_process_shop_order_meta', array( &$this, 'process_shop_ordermeta' ), 99, 2 );
		add_filter( 'woocommerce_admin_billing_fields',  array( &$this,'display_order_custom_fields' )); 				        //Admin order details page
		add_action( 'woocommerce_after_order_itemmeta', array( &$this, 'display_order_item_approval_area' ), 10, 3 );           //Product rejetion
		add_filter( 'woocommerce_hidden_order_itemmeta', array( &$this, 'hide_private_metakeys' )); 					
		add_filter( 'woocommerce_admin_html_order_item_class', array( &$this, 'manage_order_item_css_class' ), 10, 3);          //Rejected product text 
		add_action( 'woocommerce_admin_order_data_after_order_details',  array( &$this,'display_refund_data' ));
	}
	function woocommerce_metaboxes() 
	{
		$screen = wc_get_container()->get( CustomOrdersTableController::class )->custom_orders_table_usage_is_enabled()
		? wc_get_page_screen_id( 'shop-order' )
		: 'shop_order';


		add_meta_box( 'woocommerce-order-approval', esc_html__('Approval - Custom data', 'woocommerce-order-approval'), array( &$this, 'render_approval_area' ), $screen , 'normal', 'default');
	}
	function display_notices()
	{
		
	}
	function display_refund_data($order)
	{
		$has_been_refunded = $order->get_meta('_wcoa_automatically_refunded');
		$refunded_error = $order->get_meta('_wcoa_automatically_refunded_error');
		$refunded_amount = $order->get_meta('_wcoa_automatically_refunded_amount');
		
		if($has_been_refunded):
		?>
		<div class="wcoa-refund-data-container">
			<p class="form-field form-field-wide">
			<span class="wcoa-refund-title"><?php esc_html_e('The order has been refunded', 'woocommerce-order-approval'); ?></span>
			<span class="wcoa-refund-msg"><?php echo sprintf(esc_html__('Refunded amount: %s', 'woocommerce-order-approval'), wc_price($refunded_amount)); ?></span>
			</p>
		</div>
		<?php
		elseif($refunded_error):
		?>
		<div class="wcoa-refund-data-container">
			<p class="form-field form-field-wide">
				<span class="wcoa-refund-error-title"><?php esc_html_e('Refund error', 'woocommerce-order-approval'); ?></span>
				<span class="wcoa-refund-error-msg"><?php echo $refunded_error; ?></span>
			</p>
		</div>
		<?php
		endif;
	}
	function render_approval_area($post_or_order_object) 
	{
		global $wcoa_order_model, $theorder;
		$order = $theorder;
		$order_id = $theorder->get_id();
		$approval_data = $wcoa_order_model->get_approval_data($order_id);
		
		wp_enqueue_style('wcoa-order-page',  WCOA_PLUGIN_PATH.'/css/admin-order-page.css');
		wp_enqueue_style('wcoa-order-page-timepickeraddon',  WCOA_PLUGIN_PATH.'/css/vendor/timepickeraddon/jquery-ui-timepicker-addon.css');
		wp_enqueue_style( 'jquery-ui-timepicker' );
		wp_enqueue_style( 'jquery-ui-datepicker' );
		
		wp_register_script( 'wcoa-order-page', WCOA_PLUGIN_PATH.'/js/admin-order-page.js', array('jquery'));
		$js_options = array(
				'security' => wp_create_nonce('wcoa_admin_order_page'),
				'order_id' => $order_id
			);
		wp_localize_script( 'wcoa-order-page', 'wcoa', $js_options );
		wp_enqueue_script( 'wcoa-order-page');
		wp_enqueue_script( 'wcoa-order-page-timepickeraddon', WCOA_PLUGIN_PATH.'/js/vendor/timepickeraddon/jquery-ui-timepicker-addon.js', array('jquery'));
		wp_enqueue_script('jquery-ui-timepicker');
		wp_enqueue_script('jquery-ui-datepicker');

		
		?>
		<div id="wcoa_approval_area">
			<label><?php esc_html_e('Date and time', 'woocommerce-order-approval') ?></label> 
			<p><?php wcoa_html_escape_allowing_special_tags(__('You can optionally specify a date and time that can be displayed inside the approval message through the <strong>[datetime]</strong> shortcode.', 'woocommerce-order-approval')); ?></p> 
			<?php $content = wcoa_get_value_if_set($approval_data , 'datetime', ""); ?>
			<input id="wcoa_datetime_picker" name="wcoa_datetime" type="text" value="<?php echo esc_attr($content); ?>"></input>
			
			<label><?php esc_html_e('Custom approval message', 'woocommerce-order-approval') ?></label> 
			<p><?php wcoa_html_escape_allowing_special_tags(__('This message will override the Approved message and Rejected message message configured through the WooCommerce Order Approval -> Text menu. You can use the special <strong>[datetime]</strong> shortcode to display the configured date and time.', 'woocommerce-order-approval')) ?></p>
			<?php 
			$content = wcoa_get_value_if_set($approval_data , 'custom_message', "");
			wp_editor( $content, "wcoa_custom_message", array( 'media_buttons' => false,
																	   'textarea_rows' => 6,
																	   'tinymce' => true,
																	   "wpautop" => false,
																	   'textarea_name'=>"wcoa_custom_message"));
			?>
		</div>
		<?php 
	}
	function process_shop_ordermeta( $order_id, $post_obj ) 
	{
		global $wcoa_order_model;
		$wcoa_order_model->save_approval_data($_POST, $order_id);
	}
	public function hide_private_metakeys($keys)
	{
		$keys[] = '_wcoa_approval';
		$keys[] = '_wcoa_original_subtotal';
		$keys[] = '_wcoa_original_subtotal_tax';
		$keys[] = '_wcoa_rejected_after_paid';
		return $keys;
	}
	function manage_order_item_css_class($class, $item, $order)
	{
		global $wcoa_order_model;
		
		if (get_class($item) != "WC_Order_Item_Product")
			return $class;
		
		if($wcoa_order_model->get_order_item_approval_data($item->get_id()) != 'approved')
		{
			//$class .=" wcoa-line-item-rejected ";
		}
		return $class;
	}
	public function display_order_item_approval_area($item_id, $item, $product)
	{
		global $theorder, $wcoa_order_model;
		$order = $item->get_order();
		
		if (get_class($item) != "WC_Order_Item_Product" )
			return; 
		
		if(file_exists ( get_theme_file_path()."/woocommerce-order-approval/backend/order-page/approval-area.php" ))
			include get_theme_file_path()."/woocommerce-order-approval/backend/order-page/approval-area.php";
		else
			include WCOA_PLUGIN_ABS_PATH.'/templates/backend/order-page/approval-area.php';
	}
	public function display_order_custom_fields($fields, $type = 'billing')
	{
		global $wcoa_order_model, $post, $theorder;
		
		if(!isset($post) && !isset($theorder))
			return $fields;
		
		$order_id = isset($theorder) ? $theorder->get_id() : $post->ID;
		$order = wc_get_order($order_id);
		$time = $wcoa_order_model->get_time_field($order);
		if($time)
		{
			$form_field = array();
			$form_field['type'] = $time == 'as_soon_as_possible' ? 'text' : 'time';
			$form_field['label'] = esc_html__('Time','woocommerce-order-approval');
			$form_field['value'] = $time == 'as_soon_as_possible' ? esc_html__('As soon as possible','woocommerce-order-approval') : $time;
			$fields["wcoa_time"] = $form_field;
		}	
		
		
		return $fields;
	}
	
	public function render_time_field($value, $order)
	{
		return $value;
	}
}
?>