<?php 
namespace WCOA\classes\com;

class Order
{
	//rplc: woocommerce-order-approval, wcoa, WCOA
	var $notice_to_show = 'none';
	var $is_checkout_order_review = false;
	var $rejected_statuses = array('rejected', 'failed', 'cancelled', 'refunded', 'checkout-draft');
	public function __construct()
	{	
		//Payment
		add_filter( 'woocommerce_order_needs_payment', array(&$this, 'order_needs_payment'), 10, 3 );  //Added in WooCommerce 4.7: it is triggered when the order is placed, and if the returned value is not the same of 'woocommerce_cart_needs_payment', the checkout fails
		add_action('woocommerce_checkout_process', array(&$this, 'checkout_update_order_review'));
		add_filter( 'woocommerce_cart_needs_payment', array(&$this, 'cart_needs_payment'), 10, 2 ); //class-wc-cart.php: checks if the order needs to be payed after it has been placed 
		add_filter( 'woocommerce_valid_order_statuses_for_payment_complete', array(&$this, 'set_valid_order_statuses_for_payment_complete'), 30, 2 ); //class-wc-order.php: 
		//
		add_filter( 'wc_order_is_editable', array(&$this, 'is_editable'), 10, 2 ); 
		// Can be used for some paymentgateway: add_filter( 'woocommerce_payment_complete_order_status', array(&$this, 'set_order_status_after_payment'), 10, 2 );  
		add_action( 'woocommerce_checkout_create_order', array( &$this,'on_checkout_order_created'), 10, 2 );
		add_filter('woocommerce_thankyou',array(&$this, 'on_thank_you_page_rendering') ,10,1); 											//Triggered by the Thank you page
		
		//Payment processing
		//INFO:   woocommerce_payment_successful_result (two parameters): triggered once the user is redirected to any payment gateway. It is not said the payment is actually performed
		//		  other hooks: woocommerce_payment_complete , woocommerce_payment_complete and woocommerce_payment_complete_order_status_$status
		//IMPORTANT, CHECK THE 'woocommerce_order_status_changed' HANDLER, BY DEFAULT THE PAYMENT CHECK IS NOW PERFORMED THERE
		add_action('woocommerce_payment_successful_result',array(&$this, 'on_checkout_payment_successful_result'), 10, 2);
		//NOT ACCURATE: add_action('woocommerce_payment_complete',array(&$this, 'on_checkout_payment_successful_result'), 10, 2); 						//class-wc-checkout.php: triggered after a succesful payment (both on creation or triggered in a second time)
		//****
		add_filter('woocommerce_checkout_no_payment_needed_redirect',array(&$this, 'on_checkout_no_payment_needed_redirect') ,10,2); 	//class-wc-checkout.php: triggered after the order has been created if it doesn't need a payment
		add_filter('woocommerce_pay_order_product_in_stock',array(&$this, 'on_pay_order_product_in_stock') ,10,3); 						//class-wc-shortcode-checkout.php
		add_filter('woocommerce_pay_order_product_has_enough_stock',array(&$this, 'on_pay_order_product_has_enough_stock') ,10,3);		//class-wc-shortcode-checkout.php
		
		//Stock 
		add_action( 'woocommerce_order_status_approved', 'wc_maybe_reduce_stock_levels' );
		add_action( 'woocommerce_order_status_rejected', 'wc_maybe_increase_stock_levels' );
		
		//Statuses considered valid to pay the order
		add_filter('woocommerce_valid_order_statuses_for_payment',array(&$this, 'valid_order_statuses_for_payment') ,10,2);
		
		//Custom status
		add_filter( 'woocommerce_register_shop_order_post_statuses', array( &$this, 'add_order_status' ) );
		add_filter( 'wc_order_statuses', array( &$this, 'add_custom_order_status' ) );
		
		//Order status change and management
		add_action( 'woocommerce_order_status_changed', array( &$this,'order_status_changed'), 10, 4 ); 
		add_filter( 'woocommerce_valid_order_statuses_for_cancel', array( &$this,'set_order_statuses_for_cancel'), 10, 2 );
		add_action( 'woocommerce_order_status_rejected', array(&$this,'refund_on_rejection'), 10, 2 );
		
		//Order status check
		add_action( 'wp_ajax_nopriv_wcoa_check_order_status', array( &$this,'ajax_check_order_status')); 
		add_action( 'wp_ajax_wcoa_check_order_status', array( &$this,'ajax_check_order_status')); 
		
		//Allows downloadable products to be included in Approval notification email
		add_filter('woocommerce_order_is_download_permitted', array( &$this,'is_download_permitted'), 10, 2);
		add_filter('woocommerce_order_is_paid_statuses', array( &$this,'manage_paid_statuses'));
		
		add_action('init', array( &$this,'check_direct_approval_request_and_other'));
		add_action('wp_print_scripts', array( &$this,'show_notice'));
		
		//Ajax 
		add_action('wp_ajax_wcoa_update_admin_order_items_table', array(&$this, 'ajax_update_admin_order_items_table')); //Triggered on the Admin order page
	}
	function show_notice()
	{
		$js_option = array ('type' => $this->notice_to_show,
							'approved_msg' => esc_html__('The order has been successfully approved!', 'woocommerce-order-approval'),
							'rejected_msg' => esc_html__('The order has been successfully rejected!', 'woocommerce-order-approval')
							);
		wp_register_script( 'wcoa-admin-notice', WCOA_PLUGIN_PATH.'/js/admin-notice-manager.js', array('jquery'));
		wp_localize_script( 'wcoa-admin-notice', 'wcoa_notice', $js_option);
		wp_enqueue_script('wcoa-admin-notice');
		$this->notice_to_show = 'none';
	}
	//invoked when for approval email links and when returning from a PayPal payment
	function check_direct_approval_request_and_other()
	{
		global $wcoa_option_model, $post, $wcoa_user_model;
		$options = $wcoa_option_model->get_options();
		$approval_workflow = wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait');
		
		if(isset($_GET['wcoa_action']) && isset($_GET['post']) && isset($_GET['order_key']))
		{
			$action = $_GET['wcoa_action'];
			$order_id = $_GET['post'];
			$order_key = $_GET['order_key'];
			$order = wc_get_order($order_id );
			
			if(!$order || $order_key != $order->get_order_key() || $order->get_status() == 'completed' || !$wcoa_user_model->is_shop_manager())
				return;
			
			
			switch($_GET['wcoa_action'])
			{
				case 'approve':
					$this->notice_to_show = 'approved';
					$order->set_status('approved', '' , true);
					$order->save();
				break;
				case 'reject':
					$this->notice_to_show = 'rejected';
					
					$order->set_status('rejected', '' , true);
					$order->save();
				break;
			}
		}
		else if(wcoa_get_value_if_set($_GET, 'cancel_order', 'false') == 'true' && wcoa_get_value_if_set($_GET, 'order', false) &&  wcoa_get_value_if_set($_GET, 'order_id', false))
		{
			$order = wc_get_order($_GET['order_id']);
			if($order && $_GET['order'] == $order->get_order_key())
			{
				$order->set_status('cancelled');
				$order->save();
			}
		}
		//Force status after returning from a payment (PayPal)
		else if($approval_workflow == 'wait_and_pay' && isset($_GET['utm_nooverride'])  && isset($_GET['key']))
		{
			$order_id = wc_get_order_id_by_order_key($_GET['key']);
			$order_key = $_GET['key'];
			$order = wc_get_order($order_id );
			
			if(!$order)
				return;
			
			if($order->get_status() == 'approved')
			{
				$order->set_status('processing');
				$order->save();
			}
		}
		
	}
	
	function ajax_check_order_status()
	{
		$order_id = wcoa_get_value_if_set($_POST, 'order_id', false);
		if($order_id)
		{
			$order = wc_get_order($order_id);
			if($order)
			{
				$this->has_to_be_automatically_cancelled($order);
				echo $order->get_status();
			}
			else echo "error";
		}
		else echo "error";
		wp_die();
	}
	function manage_paid_statuses($statuses)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'pay_and_wait')
		{
			$statuses[] = 'approved';
			$statuses[] = 'rejected';
			
			if(!wcoa_get_value_if_set($options, array('approval_workflow','exclude_waiting_for_approval_from_paid_statuses'), false) )
				$statuses[] = 'approval-waiting';
		}
		return $statuses;
	}
	function is_download_permitted($can_be_downloaded, $wc_order)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'pay_and_wait')
		{
			if($wc_order->has_status( 'approved' ))
				$can_be_downloaded = true;
		}	
		
		return $can_be_downloaded;
	}
	function set_order_statuses_for_cancel( $statuses, $order )
	{
		if(isset($_GET['wcoa_cancel']))
			$statuses[] = $order->get_status();
		
		return $statuses;
	}
	public function get_lang($order)
	{
		global $wcoa_wpml_model;
		
		$wc_order = is_object($order) ? $order : wc_get_order($order);
		$default = $wcoa_wpml_model->get_current_language(); //exameple en;

		if(!isset($order) || $order == false)
			return $default;
		
		$result = $wc_order->get_meta('wpml_language');
		$result = !$result || !is_string($result) || $result == "" ? $default : $result;
		
		return strtolower($result);
	}
	function add_order_status( $order_statuses )
	{
    
	    // Status must start with "wc-"
	   $order_statuses['wc-approval-waiting'] = array(                                 
	   'label'                     => esc_html__( 'Waiting for approval', 'Order status', 'woocommerce-order-approval' ),
	   'public'                    => true,                                 
	   'exclude_from_search'       => false,                                 
	   'show_in_admin_all_list'    => true,                                 
	   'show_in_admin_status_list' => true,                                 
	   'label_count'               => _n_noop( 'Waiting for approval <span class="count">(%s)</span>', 'waiting for approval <span class="count">(%s)</span>', 'woocommerce-order-approval' ),                              
	   );      
	   $order_statuses['wc-approved'] = array(                                 
	   'label'                     => esc_html__( 'Approved', 'Order status', 'woocommerce-order-approval' ),
	   'public'                    => true,                                 
	   'exclude_from_search'       => false,                                 
	   'show_in_admin_all_list'    => true,                                 
	   'show_in_admin_status_list' => true,                                 
	   'label_count'               => _n_noop( 'Approved <span class="count">(%s)</span>', 'Approved <span class="count">(%s)</span>', 'woocommerce-order-approval' ),                              
	   ); 
		$order_statuses['wc-rejected'] = array(                                 
	   'label'                     => esc_html__( 'Rejected', 'Order status', 'woocommerce-order-approval' ),
	   'public'                    => true,                                 
	   'exclude_from_search'       => false,                                 
	   'show_in_admin_all_list'    => true,                                 
	   'show_in_admin_status_list' => true,                                 
	   'label_count'               => _n_noop( 'Rejected <span class="count">(%s)</span>', 'Rejected <span class="count">(%s)</span>', 'woocommerce-order-approval' ),                              
	   );	   
	   return $order_statuses; 
	
	}
	
	function add_custom_order_status( $order_statuses ) 
	{      
	   $order_statuses['wc-approval-waiting'] = esc_html__( 'Waiting for approval', 'woocommerce-order-approval' );       
	   $order_statuses['wc-approved'] = esc_html__( 'Approved', 'woocommerce-order-approval' );       
	   $order_statuses['wc-rejected'] = esc_html__( 'Rejected', 'woocommerce-order-approval' );       
	   return $order_statuses;
	}
	//End custom status

	
	public function save_approval_data($posted_data, $order_id)
	{
		$order = wc_get_order($order_id);
		if(!isset($order) || $order == false)
			return;
		
		$items_to_refund = array();
		$order->update_meta_data('wcoa_custom_message', stripslashes(wcoa_get_value_if_set($posted_data, 'wcoa_custom_message', "")));
		$order->update_meta_data('wcoa_datetime', wcoa_get_value_if_set($posted_data, 'wcoa_datetime', ""));
		
		/*$items_approval = wcoa_get_value_if_set($_POST, 'wcoa_item_approval', array());
		foreach($items_approval as $item_id => $item_approval_data)
		{
			$order_item = $order->get_item($item_id);
			$this->update_order_item_meta($item_id, "_wcoa_approval", $item_approval_data);
			//$order_item->update_meta_data("_wcoa_approval", $item_approval_data);
			
			if($item_approval_data == 'rejected')
			{
				$items_to_refund[$item_id] = $order_item->get_total();
				$order_item->update_meta_data("_line_total", 0);
			}
			else if($item_approval_data == 'approved')
			{
				if($order_item->get_total() == 0) //Tries to restore the price
				{
					 $product = $order_item->get_product();
					$price = $order_item->get_quantity() * $product->get_price();
					$order_item->set_subtotal($price);
					$order_item->save(); 
				}
			}
			
			$order_item->save();
		}
		//Update order according the removed items
		if(!empty($items_to_refund))
		{
			$order->calculate_shipping();
			$order->calculate_taxes();
			$order->calculate_totals();
		}*/
		
		$order->save();
	}
	public function ajax_update_admin_order_items_table()
	{
		$item_id = filter_input( INPUT_POST, 'id', FILTER_VALIDATE_INT );
		$order_id = filter_input( INPUT_POST, 'order_id', FILTER_VALIDATE_INT );
		$value = filter_input( INPUT_POST, 'value', FILTER_SANITIZE_STRING );
		$order = wc_get_order($order_id);
		$result = array('total'=> 0);
		if(wp_verify_nonce( wcoa_get_value_if_set($_POST, 'security', ""), 'wcoa_admin_order_page' ) && $order)
		{
			$order_item = $order->get_item($item_id);
			$this->update_order_item_meta($item_id, "_wcoa_approval", $value);
			//Taxes management
			/*$order_taxes = $order->get_taxes();
			$tax_data = wc_tax_enabled() ? $order_item->get_taxes() : false;
			$tax_original_subtotal = 0;
			foreach ( $order_taxes as $tax_item ) 
			{
				$tax_item_id       = $tax_item->get_rate_id();
				$tax_item_total    = isset( $tax_data['total'][ $tax_item_id ] ) ? $tax_data['total'][ $tax_item_id ] : 0;
				$tax_item_subtotal = isset( $tax_data['subtotal'][ $tax_item_id ] ) ? $tax_data['subtotal'][ $tax_item_id ] : 0;
				$tax_original_subtotal += $tax_item_subtotal;
			}*/
			//
			if($value == 'rejected')
			{
				$this->update_order_item_meta($item_id, "_wcoa_original_subtotal", $order_item->get_subtotal());
				$this->update_order_item_meta($item_id, "_wcoa_original_subtotal_tax", $order_item->get_subtotal_tax());
				$this->update_order_item_meta($item_id, "_wcoa_rejected_after_paid", $order->is_paid());
				$order_item->set_subtotal(0);
				$order_item->save(); 
				$result['total'] = 0;
			}
			else if($value == 'approved')
			{
				$subtotal = $this->get_order_item_meta($item_id, "_wcoa_original_subtotal");
				$order_item->set_subtotal($subtotal);
				$order_item->save(); 
				$result['total'] = $subtotal;
			}
			
			/* $order->calculate_shipping();
			$order->calculate_taxes();
			$order->calculate_totals(); */
		}
		echo json_encode($result);
		wp_die();
	}
	public function update_order_item_meta($item_id, $key, $value)
	{
		return wc_update_order_item_meta($item_id, $key, $value);
	}
	public function get_order_item_meta($item_id, $key, $single = true)
	{
		return wc_get_order_item_meta($item_id, $key, $single);
	}
	public function get_order_item_approval_data($item_id, $order = null )
	{
		$value = $this->get_order_item_meta($item_id, "_wcoa_approval");
		/* $order_item = $order->get_item($item_id);
		$value = $order_item->get_meta("_wcoa_approval"); */
		return $value ? $value : 'approved';
	}
	public function get_approval_data($order_id, $replace_shortcode = false)
	{
		$order = wc_get_order($order_id);
		$result = array('wcoa_custom_message' => "", 'wcoa_datetime' =>"");
		if(!isset($order) || $order == false)
			return $result;
		
		$result['custom_message'] = $order->get_meta('wcoa_custom_message');	
		$result['datetime'] = $order->get_meta('wcoa_datetime');	
		if($replace_shortcode)
		{
			$date = date_create($result['datetime']);
			$date_time = date_format($date,get_option('date_format')." ".get_option('time_format'));
			$result['custom_message'] = str_replace('[datetime]', $date_time, $result['custom_message']);
		}
		return $result;
	}
	public function is_editable($is_editable, $order)
	{
		return $order->get_status() == 'approval-waiting' ? true : $is_editable;
	}
	public function checkout_update_order_review($posted_data)
	{
		$this->is_checkout_order_review = true;
	}
	public function order_needs_payment($order_needs_payment, $order, $valid_order_statuses)
	{
		global $wcoa_option_model, $wcoa_user_model;
		$options = $wcoa_option_model->get_options();
		
		//Automatic approval check (for now it is not needed)
		$selected_roles = wcoa_get_value_if_set($options, array('automatic_approval','user_role'), array());
		$automatic_approval_by_products = $this->automatic_approval_according_products($order, $options);
		$automatic_approva_user_roles = $wcoa_user_model->belogs_to_roles($selected_roles);
		
		if($this->is_checkout_order_review) //During the checkout
		{
			return  wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'pay_and_wait';
		}
		//When accessing the Order details page: if the order contains products that automatically approve the order or the user belongs to roles that automatically approve the order, it means the order has been already paid during the checkout
		/*  Uncomment to unlock the mechanism
			else if($automatic_approval_by_products || $automatic_approva_user_roles)
				return false; */
				
		return $order_needs_payment;
	}
	public function cart_needs_payment($cart_needs_to_be_payment, $cart = null ) //$istance is WC_Cart obj?
	{
		global $wcoa_option_model,$wcoa_user_model;
		$options = $wcoa_option_model->get_options();
		
		
		/* 
		 $selected_roles = wcoa_get_value_if_set($options, array('automatic_approval','user_role'), array());
		 $automatic_approval_by_products = $cart ? $this->automatic_approval_according_products($cart, $options) : false;
		 $automatic_approva_user_roles = $wcoa_user_model->belogs_to_roles($selected_roles);
		 if($automatic_approval_by_products || $automatic_approva_user_roles)
			return true;
		 */
		
		return wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'pay_and_wait' ;
	}
	public function set_valid_order_statuses_for_payment_complete($valid_statuses, $order)
	{
		global $wcoa_option_model,$wcoa_user_model;
		$options = $wcoa_option_model->get_options();
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') != 'pay_and_wait')
		{
			if (($key = array_search($order->get_status(), $valid_statuses)) !== false) 
				unset($valid_statuses[$key]);
		}
		
		return $valid_statuses;
	}
	public function valid_order_statuses_for_payment($statuses, $order = null)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		if(wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'wait_and_pay')
			$statuses[] = 'approved';
		
		return $statuses;
	}
	public function on_pay_order_product_has_enough_stock($stock_quantity, $product, $order)
	{
		global $wcoa_option_model;
		
		//Stock check
		$hold_stock_minutes = (int) get_option( 'woocommerce_hold_stock_minutes', 0 );
		$quantities = array();
		foreach ( $order->get_items() as $item_key => $item ) 
		{
			if ( $item && is_callable( array( $item, 'get_product' ) ) ) 
			{
				$product = $item->get_product();

				if ( ! $product ) {
					continue;
				}

				$quantities[ $product->get_stock_managed_by_id() ] = isset( $quantities[ $product->get_stock_managed_by_id() ] ) ? $quantities[ $product->get_stock_managed_by_id() ] + $item->get_quantity() : $item->get_quantity();
			}
		}
		$held_stock     = ( $hold_stock_minutes > 0 ) ? wc_get_held_stock_quantity( $product, $order->get_id() ) : 0;
		$required_stock = $quantities[ $product->get_stock_managed_by_id() ];
		//
					
		$options = $wcoa_option_model->get_options();
		$has_to_be_approved = wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'wait_and_pay';
		$stock_quantity = $has_to_be_approved == 'wait_and_pay' ? $product->get_stock_quantity() >= 0 /* $product->get_stock_quantity() >= ( $held_stock + $required_stock ) */  : $stock_quantity;
		
		return $stock_quantity;
	}
	public function on_pay_order_product_in_stock($is_in_stock, $product, $order)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		$has_to_be_approved = wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'wait_and_pay';
		$is_in_stock = $has_to_be_approved == 'wait_and_pay' ? true : $is_in_stock;
			
		return $is_in_stock;
	}
	public function on_checkout_order_created($order, $data)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		$has_to_be_approved = wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'wait_and_pay';
		$order->update_meta_data('wcoa_workflow_type', wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait'));
		$order->save();
		
		//In case of "Wait and Pay" the new order notification email (to admin) and the "order received" might be not sent. This forces the sending (the WC_Email_New_Order()->get_new_order_email_sent() checks if it already sent)
		if(wcoa_get_value_if_set($options, array('approval_workflow','type'), 'wait_and_pay') == 'wait_and_pay')
		{
			$disable = wcoa_get_value_if_set($options, array('notification','disable_forced_notification'), false) == "true";
			if(!$disable)
			{
				WC()->mailer()->emails['WC_Email_New_Order']->trigger( $order->get_id(), $order);
				WC()->mailer()->emails['WC_Email_Customer_On_Hold_Order']->trigger( $order->get_id(), $order);
			}
		}
		
		/*if($has_to_be_approved) //It has to be approved first, so no stock reduction
			$this->restore_order_stock($order->get_id(), $order, 'increase'); //avoid unnecessary stock reduction*/
	}
	public function can_be_automatically_approved($order)
	{
		global $wcoa_option_model, $wcoa_user_model;
		$options = $wcoa_option_model->get_options();
		
		//automatic approval per payment gateway
		$automatic_approval_payment_gateway = wcoa_get_value_if_set($options, array('automatic_approval','payment_gateways', $order->get_payment_method()), false); 
	
		//automatic approval per shipping
		$automatic_approval_shipping_method  = false;
		
		//On thank you page, if the order fails, it won't be automatically approved
		if($this->is_bad_status($order))
			return false;
		
		if(count($order->get_shipping_methods()) > 0)
		{
			$first_method = array_values($order->get_shipping_methods())[0]; //NOTE: there could be more than 1 shipping
			$shipping_method_id = $first_method ->get_method_id().":".$first_method ->get_instance_id(); 
			$automatic_approval_shipping_method = wcoa_get_value_if_set($options, array('automatic_approval','shipping_method', $shipping_method_id), false);  	
		}
		//automatic approval per product  
		$automatic_approval_by_products = $this->automatic_approval_according_products($order, $options);
		
		//roles 
		$selected_roles = wcoa_get_value_if_set($options, array('automatic_approval','user_role'), array());
		$automatic_approva_user_roles = $wcoa_user_model->belogs_to_roles($selected_roles);
		
		//order subtotal
		$value = wcoa_get_value_if_set($options, array('automatic_approval','order_total'), "");
		$operator = wcoa_get_value_if_set($options, array('automatic_approval','order_total_operator'), "more");
		$automatic_approval_order_subtotal = false;
		if($value != 0 && $value  != "")
			$automatic_approval_order_subtotal = $operator == 'more' ? $value < $order->get_subtotal() : $value > $order->get_subtotal(); //sum of the products excluding taxes
		
		//coupon 
		$coupons = $order->get_coupon_codes();
		$automatic_approval_coupon = false;
		foreach((array)$coupons as $coupon_id)
		{
			if($automatic_approval_coupon)
				break;
			$coupon    = new \WC_Coupon( $coupon_id );
			$automatic_approval_coupon = $coupon->get_meta( 'wcoa_automatic_approval' ) ? $coupon->get_meta( 'wcoa_automatic_approval' ) : false;
		}
		
		if($automatic_approval_coupon || $automatic_approval_payment_gateway || $automatic_approval_shipping_method || $automatic_approval_by_products || $automatic_approval_order_subtotal || $automatic_approva_user_roles)
		{
			$order->update_meta_data('wcoa_skip_email_notification',  true);
			$order->save();
			return true;
		}
		$order->update_meta_data('wcoa_skip_email_notification',  false);
		return false;
	}
	public function on_thank_you_page_rendering($order_id)
	{
		global $wcoa_option_model;
		$order = wc_get_order($order_id);
		$options = $wcoa_option_model->get_options();
		
		if(apply_filters('wcoa_before_order_status_update', false, 'woocommerce_thankyou', $order))
			return;
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'disable'), false) || wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'management_method'), "default") != "thank_you")
			return;
		
		if(!isset($order) || is_bool($order))
			return;
		
		$already_checked = $order->get_meta('_wcoa_first_status_check');
		if($already_checked)
			return;
		
		$order->add_meta_data('_wcoa_first_status_check', true, true);
		$order->save();
		
		$can_be_automatically_approved = $this->can_be_automatically_approved($order);
		
		$order->update_status(apply_filters('wcoa_thank_you_page_rendering',!$can_be_automatically_approved ? 'approval-waiting' : 'approved', $order, $can_be_automatically_approved), '', true);
	}
	public function on_checkout_payment_successful_result($result, $order_id)
	{
		global $wcoa_option_model, $wcoa_email_model;
		$order = wc_get_order($order_id);
		$options = $wcoa_option_model->get_options();
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'wait_and_pay')
		{
			$wcoa_email_model->send_email($order, "order-paid", "admin");
		}
		
		//wait and pay: if the payment has been performed, this code is executed
		if(did_action('woocommerce_checkout_create_order')  == 0 ) //Action triggred when the order is created after checkout
		{
			$result['redirect'] = add_query_arg('wcoa_order_paid_redirection', true, $result['redirect']);
			return $result;
		}
	
		if(!isset($order) || is_bool($order))
			return $result;
		
		if(apply_filters('wcoa_before_order_status_update', false, 'woocommerce_payment_successful_result', $order))
			return $result;
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'disable'), false) || wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'management_method'),  "default") == 'thank_you')
			return $result;
		
		$can_be_automatically_approved = $this->can_be_automatically_approved($order);
		$order->update_status(apply_filters('wcoa_on_checkout_payment_successful',!$can_be_automatically_approved ? 'approval-waiting' : 'approved', $order, $can_be_automatically_approved), '', true);
		return $result;
	}	
	public function on_checkout_no_payment_needed_redirect($url, $order)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		
		if(did_action('woocommerce_checkout_create_order') == 0) //Action triggred when the order is created after checkout
			return $url;
		
		if(apply_filters('wcoa_before_order_status_update', false, 'woocommerce_checkout_no_payment_needed_redirect', $order))
			return $url;
		
		if(wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'disable'), false) || wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'management_method'),  "default") == "thank_you")
			return $url;
		
		$can_be_automatically_approved = $this->can_be_automatically_approved($order);
		$order->update_status(apply_filters('wcoa_on_checkout_no_payment_needed',!$can_be_automatically_approved ? 'approval-waiting' : 'approved', $order, $can_be_automatically_approved), '', true);
		return $url;
	}
	public function order_status_changed( $this_get_id, $this_status_transition_from, $this_status_transition_to, $order )
	{
		
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		
		//Payments
		if($this_status_transition_from == "pending" && !in_array($this_status_transition_to, array('failed', 'cancelled')))
		{
			if(apply_filters('wcoa_before_order_status_update', false, 'order_status_changed_on_payment', $order))
				return;
			
			if(!wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'disable'), false) && wcoa_get_value_if_set($options, array('approval_workflow','order_status_assignment', 'management_method'), "default") == "default")
			{
				$can_be_automatically_approved = $this->can_be_automatically_approved($order);
				$order->update_status(apply_filters('wcoa_on_checkout_payment_successful',!$can_be_automatically_approved ? 'approval-waiting' : 'approved', $order, $can_be_automatically_approved), '', true);
				return;
			}
		}
		//INFO: in case of  wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait') == 'wait_and_pay' 
		//		$this_status_transition_from is "approved" and $this_status_transition_to == "hold" (if paid) of "cancelled" if the payment is cancelled
		//
		
		if( $this_status_transition_from == 'approval-waiting' && ($this_status_transition_to == 'rejected' || $this_status_transition_to == 'approved'))
		{
			$order->update_meta_data('wcoa_status_chage_date',  current_time( 'mysql' ));
			$order->save();
		}
		else if($this_status_transition_to == 'approval-waiting' )
		{
			$order->update_meta_data('wcoa_status_chage_date',  "");
			$order->save();
		}
		
	}
	//Handler to manage the order refund (according the special option)
	public function refund_on_rejection($order_id, $order)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		$auotomatically_refund = wcoa_get_value_if_set($options, array('approval_workflow','refund_on_rejection'), false) == "true";
		//debug: $order->delete_meta_data('_wcoa_automatically_refunded');
		
		$already_refunded = $order->get_meta('_wcoa_automatically_refunded');
		if(!$auotomatically_refund || !$order || $already_refunded)
			return;
		
		$refund = wc_create_refund(array(
											'amount'         => $order->get_total(),
											'order_id'       => $order_id,
											'refund_payment' => true,
											'restock_items'  => true,
										));
			
		if ( is_wp_error($refund) )
		{
			$order->update_meta_data('_wcoa_automatically_refunded_error', $refund->get_error_message());
			wcoa_write_log($refund->get_error_message());
		}
		else
		{
			$order->update_meta_data('_wcoa_automatically_refunded',  true);
			$order->update_meta_data('_wcoa_automatically_refunded_amount', $refund);
		}
		$order->save();
		
	}
	//approval-waiting || approved || rejected
	public function get_approval_status($order)
	{
		if($this->is_approval_waiting($order))
		{
			return 'approval-waiting';
		}
		
		return $this->is_approved($order) ? 'approved' : 'rejected';
	}
	public function is_approval_waiting($order)
	{
		global $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		
	    $custom_waiting_for_approva_status = wcoa_get_value_if_set($options, array('approval_workflow', 'order_statuses_considered_as_witing_for_approval',$order->get_status()), false);
		if($custom_waiting_for_approva_status)
				return true;
			
		$workflow_type = wcoa_get_value_if_set($options, array('approval_workflow','type'), 'pay_and_wait');
		return $order->get_status() == 'approval-waiting' || ( $workflow_type == 'pay_and_wait' && $order->get_status() == 'pending');
	}
	public function get_rejected_statuses()
	{
		return $this->rejected_statuses;
	}
	public function get_order_statuses_that_could_be_considered_as_waiting_for_approval()
	{
		$statuses = wc_get_order_statuses();
		foreach($statuses as $status_code => $status_name)
		{
			$current_status = str_replace("wc-","",$status_code);
			if(!in_array($current_status, $this->rejected_statuses) && $current_status != 'approval-waiting' && $current_status != 'approved')
				$result[$current_status] = $status_name;
		}
		return $result;
	}
	public function is_rejected($order)
	{
		return in_array($order->get_status(),$this->rejected_statuses );
	}
	public function is_bad_status($order)
	{
		$bad_statuses = array('failed', 'cancelled', 'refunded');
		return in_array($order->get_status(),$bad_statuses );
	}
	public function is_approved($order)
	{
		global $wcoa_option_model;	
		return !$this->is_rejected($order) && $order->get_status() != 'approval-waiting';
	}
	public function get_creation_date($order)
	{
		return $order->get_date_created()->format(get_option('date_format')." ".get_option('time_format'));
	}
	public function get_approval_date($order, $raw = false)
	{
		$date = $order->get_meta('wcoa_status_chage_date');
		if($raw)
			return $date;
		
		if($date)
		{
			$date = date_create($date);
			$date = date_format($date, get_option('date_format')." ".get_option('time_format'));
		}
		else 
			$date = "";
		return $date;
	}
	function get_time_field($order)
	{
		return $order ? $order->get_meta('_billing_wcoa_time') : false;
	}
	function has_to_be_automatically_cancelled($order, $cancel_if_needed = true)
	{
		global $wcoa_time_model, $wcoa_option_model;
		$options = $wcoa_option_model->get_options();
		$date = $this->get_approval_date($order, true);
		$offset = wcoa_get_value_if_set($options, array('approval_workflow','automatic_cancellation_time'), 0);
		$result = $offset != 0;
		if($offset > 0 && $cancel_if_needed && ($order->get_status() == 'approved' || $order->needs_payment()) && 
		   $this->is_approved($order) && $date && $wcoa_time_model->time_compare($date, $offset) == 1)
		{
			$order->set_status('cancelled', '' , true);
			$order->save();
		}
		return $result;
	}
	function can_be_cancelled($order)
	{
		global $wcoa_time_model, $wcoa_option_model;
		$date = $this->get_approval_date($order, true);
		$options = $wcoa_option_model->get_options();
		$offset = wcoa_get_value_if_set($options, array('approval_workflow','cancellation_time'), 0);
		
		return $order->get_status() != 'cancelled' && $this->is_approved($order) && $date && $offset > 0 && $wcoa_time_model->time_compare($date, $offset) != 1;
	}
	function get_order_details_page_url($order)
	{
		$order_url = $order->get_customer_id() ? $order->get_view_order_url(): 
												 $order->get_checkout_order_received_url( ); 
												 
		return $order_url;
	}
	 public function restore_order_stock( $order_id, $order, $operation = 'increase' ) //Type of opertion, allows 'set', 'increase' and 'decrease'.
	 {
		$items = $order->get_items();

		if (  count( $items ) > 0 )
				return; // We exit 
		
		foreach ( $order->get_items() as $item ) 
		{
			$product_id = $item->get_product_id();

			if ( $product_id > 0 ) {
				$product = $item->get_product();

				if ( $product && $product->exists() && $product->managing_stock() ) 
				{

					// Get the product initial stock quantity (before update)
					$initial_stock = $product->get_stock_quantity();

					$item_qty = apply_filters( 'woocommerce_order_item_quantity', $item->get_quantity(), $this, $item );

					// Update the product stock quantity
					// Replace DEPRECATED methods: increase_stock() & discrease_stock()
					wc_update_product_stock( $product, $item_qty, $operation );

					// Get the product updated stock quantity
					$updated_stock = $initial_stock + $item_qty;

					//do_action( 'woocommerce_auto_stock_restored', $product, $item );

					// A unique Order note: Store each order note in an array…
					$order_note[] = sprintf( esc_html__( 'Product ID #%s stock incremented from %s to %s.', 'woocommerce' ), $product_id, $initial_stock, $updated_stock);

				}
			}
		}
		
	}
	
	public function automatic_approval_according_products($order_or_cart, $options)
	{
		global $wcoa_wpml_model;
		$workflow_type = wcoa_get_value_if_set($options, array('automatic_approval','by_product', 'workflow_type'), 'manual_approval'); // 'manual_approval' || 'automatic_approval'
		$selected_products = wcoa_get_value_if_set($options, array('automatic_approval','by_product', 'product'), array());
		$selected_categories = wcoa_get_value_if_set($options, array('automatic_approval','by_product', 'category'), array());
		$selected_tags = wcoa_get_value_if_set($options, array('automatic_approval','by_product', 'tag'), array());
		$consider_subcategories = wcoa_get_value_if_set($options, array('automatic_approval','by_product', 'consider_subcategories'), false);
		
		$items = get_class($order_or_cart) != 'WC_Cart' ? $order_or_cart->get_items() : $order_or_cart->get_cart();
		$is_present = false;
		$approve = false;
		
		if(empty($selected_products) && empty($selected_categories) && empty($selected_tags))
			return false;
		
		foreach ( $items as $item ) 
		{
			if($is_present)
				break;
			
			if(!is_array($item))
			{
				$product_id = $item->get_product_id();
				$product_variation_id = $item->get_variation_id();
			}
			else 
			{
				$product_id = $item['product_id'];
				$product_variation_id = $item['variation_id'];
			}
			
			$product_item = wc_get_product($product_id);
			if(!$product_item)
				continue;
			
			$category_ids = $product_item->get_category_ids();
			$tags_ids = $product_item->get_tag_ids();
			
			//translations
			$product_id = $wcoa_wpml_model->get_original_id($product_id);
			$product_variation_id = $product_variation_id != 0 ?  $wcoa_wpml_model->get_original_id($product_variation_id) : $product_variation_id;
			
			$main_category_ids = array();
			foreach($category_ids as $cat_id)
				$main_category_ids[] = $wcoa_wpml_model->get_main_language_id($cat_id, 'product_cat');
			
			$main_tags_ids = array();
			foreach($tags_ids as $tag_id)
				$main_tags_ids[] = $wcoa_wpml_model->get_main_language_id($tag_id, 'product_tag');			
				
			if($consider_subcategories && !empty($main_category_ids))
			{
				$sub_categories = array();
				foreach($selected_categories as $cat_id)
				{
					$args = array(
						'type'                     => 'product',
						'child_of'                 => $cat_id,
						'parent'                   => '',
						'orderby'                  => 'name',
						'order'                    => 'ASC',
						'hide_empty'               => 1,
						'hierarchical'             => 1,
						'exclude'                  => '',
						'include'                  => '',
						'number'                   => '',
						'taxonomy'                 => "product_cat",
						'pad_counts'               => false

					); 
					$result = get_categories( $args );
					
					foreach((array)$result as $sub_cat)
					{
						$sub_categories[] =  $wcoa_wpml_model->get_main_language_id($sub_cat->term_id, 'product_cat') ;
					}
				}
				
				$selected_categories = !empty($sub_categories) ? array_merge($selected_categories, $sub_categories) : $selected_categories;
			}
			
			$is_present = in_array($product_id, $selected_products) || ($product_variation_id != 0 && in_array($product_variation_id, $selected_products));
			if(!$is_present && !empty($selected_categories) && !empty($main_category_ids))
			{
				$result = array_intersect($main_category_ids, $selected_categories);
				$is_present = !empty($result);
			}
			if(!$is_present && !empty($selected_tags) && !empty($main_tags_ids))
			{
				$result = array_intersect($main_tags_ids, $selected_tags);
				$is_present = !empty($result);
			}
		}
		
		if($workflow_type == 'manual_approval')
			$approve = $is_present ? false : true;
		else if($workflow_type == 'automatic_approval')
			$approve = $is_present ? true : false;
		
		
		return $approve;
		
	}
}
?>